require "daemons"

module Sidewatch

  class Daemon

    attr_accessor :sidekiq
    attr_accessor :cloudwatch
    attr_accessor :interval

    def initialize(config = {})
      sidekiq_config = config.fetch(:sidekiq){ Hash.new }.symbolize_keys
      self.sidekiq = Sidewatch::Sidekiq.new(sidekiq_config)
      cloudwatch_config = config.fetch(:cloudwatch){ Hash.new }.symbolize_keys
      self.cloudwatch = Sidewatch::Cloudwatch.new(cloudwatch_config)
      self.interval = config.fetch(:interval){ 60 }
    end

    def run
      logger.info("Starting #{self.class}...")
      every(interval) do
        metrics = sidekiq.metrics
        logger.info(metrics)
        cloudwatch.send(metrics)
      end
    end

    private

    def every(interval, &block)
      loop do
        ts = Time.now
        block.call
        remaining = (ts + interval) - Time.now
        sleep remaining
      end
    rescue Interrupt
      logger.info("Interrupted; exiting.")
    end

    def logger
      Sidewatch.logger
    end

  end
end
